(*
 *  The main program.  
 *
 *  All the interesting parts of the system are elsewhere.  This module is
 *  responsible only for parsing the command line, setting up various things,
 *  parsing files, handling imported files, and calling Toplevel.do_command
 *  for the rest.
 *)

open Format

let parseArgs () =
  let inFile = ref (None : string option) in
  Arg.parse [
    "-set", Arg.String (Support.Flags.setFlag Support.Error.unknown true), "Set a flag";
    "-reset", Arg.String (Support.Flags.setFlag Support.Error.unknown false), "Reset a flag";
    "-do", Arg.String (fun c -> Support.Flags.doCmd Support.Error.unknown c []),
      "Execute an internal command";
    "-I", Arg.String (fun f -> Support.Flags.doCmd Support.Error.unknown "appendToPath" [f]),
      "Append a directory to the search path"
  ] (fun s -> match !inFile with
                Some(_) -> Support.Error.err "You must specify exactly one input file"
              | None -> inFile := Some(s)) "";
  match !inFile with
    None -> Support.Error.err "You must specify an input file"
  | Some(s) -> s

let parseFile inFile =
  let pi =
    try open_in inFile with Sys_error m -> 
    Support.Error.err ("Could not open " ^ inFile ^ " (" ^ m ^ ")")
  in let lexbuf = Lexer.create inFile pi
  in let result =
    try Parser.toplevel Lexer.main lexbuf with Parsing.Parse_error -> 
    Support.Error.errAt (Lexer.info lexbuf) "Parse error"
in
  Parsing.clear_parser(); close_in pi; result

let trap_exceptions = 
  Support.Flags.createFlag "trapexceptions"
    "Continue after a typechecking or evaluation error"
    false

let rec toplevel_loop ctx imports c =
  open_vbox 0;
  match c with
      [] -> ()
    | (cmd::rest) -> 
        (match cmd with
           Exp.Import(f) -> 
             print_flush();
             if Support.StringSet.mem f imports
               then toplevel_loop ctx imports rest
               else toplevel_file ctx imports f rest
         | Exp.Set(fi,f,b) -> 
             Support.Flags.setFlag fi b f;
             print_flush();
             toplevel_loop ctx imports rest
         | Exp.Do(fi,c,args) -> 
             Support.Flags.doCmd fi c args;
             print_flush();
             toplevel_loop ctx imports rest
         | _ ->
             open_hvbox 0;
             let ctx' = 
               if !trap_exceptions then
                 try Toplevel.do_command ctx cmd 
                 with Support.Error.Exit(_) -> ctx
               else
                 Toplevel.do_command ctx cmd 
             in print_flush(); toplevel_loop ctx' imports rest)

and toplevel_file ctx imports f rest =
  let commands = parseFile f in 
  toplevel_loop ctx (Support.StringSet.add f imports) (commands@rest)

let main () = 
  let inFile = parseArgs() in
  toplevel_file Toplevel.initial_context Support.StringSet.empty inFile []

let () = set_max_boxes 1000
let () = set_margin 100
let res = 
  Printexc.catch (fun () -> 
    try main();0 
    with Support.Error.Exit x -> x) 
  ()
let () = print_flush()
let () = exit res



