(* 
 * The module Support collects a number of low-level facilities used by
 * the other modules in the typechecker/evaluator.
 * 
 * Several sub-modules are provided:
 *   Error      --  Error message printing
 *   Flags      --  Support for named flags and switches
 *   StringSet  --  A "ready-built" module of sets of strings
 *   StringMap  --  A "ready-built" module of maps with string keys
 *)


module Error : sig

  (*
   * Command line parsing and error handling.
   *)

  exception Exit of int

  (* An element of the type info represents a "file position": a 
     file name, line number, and character position within the line.  
     Used for printing error messages. *)
  type info
  val unknown : info
  val progName : info

  (*
   * Create file position info: filename lineno column
   *)
  val create : string -> int -> int -> info
  val printInfo : info -> unit

  (*
   * Print an error message and fail.  The printing function is called
   * in a context where the formatter is processing an hovbox.  Insert
   * calls to Format.print_space to print a space or, if necessary,
   * break the line at that point.
   *)
  val errf : (unit->unit) -> 'a
  val errfAt : info -> (unit->unit) -> 'a

  val err : string -> 'a
  val errAt : info -> string -> 'a

  val bug : string -> 'a
  val bugf : (unit->unit) -> 'a
  val bugfAt : info -> (unit->unit) -> 'a
  val bugAt : info -> string -> 'a

  val warning : string -> unit
  val warningAt : info -> string -> unit

  val notImplemented : string -> 'a

  type 'a withinfo = {i: info; v: 'a}

  val printErrorsAsIfInFile: string -> unit

end

(* ------------------------------------------------------------------------ *)

module Flags : sig

  (*
   * Flags and commands, accessible dynamically by name.
   *)

  val createFlag : string -> string -> bool -> bool ref
  val createFlagIfEnabled : string -> string -> bool -> bool -> bool ref
  val setFlag : Error.info -> bool -> string -> unit

  val createCmd : string -> string -> (Error.info -> string list -> unit) 
                  -> unit
  val doCmd : Error.info -> string -> (string list) -> unit

end


(* ------------------------------------------------------------------------ *)

(*
 * Ready-build set and map implementations
 *)

module StringSet : Set.S with type elt = string
module StringMap : Map.S with type key = string

val assoc3 : 'a -> ('a * 'b * 'c) list -> ('b * 'c)
val mem_assoc3 : 'a -> ('a * 'b * 'c) list -> bool



